#ifndef __VADCLIENT_AUDIO_H__
#define __VADCLIENT_AUDIO_H__


/* seconds of audio to buffer before playback starts */
#define SECS_TO_BUFFER  5		


struct audio_device {

	int fd_dsp;			/* File descriptor of the device     */
	struct audio_params dev_params;	/* Audio parameters of the device    */	
	int blocksize;			/* Blocksize of the device	     */
	int set_flag;			/* = 1 when params are set, 0 if not */

};

/* Opens the audio device */
int audio_open();

/* Closes the audio device */
void audio_close();

/* Closes and reopens the audio device, then sets parameters */
void audio_set_params(struct audio_params *ap);

/* write the buffered audio data to the device for playback */
int write_to_audio(struct ringbuffer *devbuf);

/* tells the writing loop to ingore synchronziation timestamps */
void set_buffer_flush();
void unset_buffer_flush();

/* Returns 1 if the dev_params are set, 0 if not */
int is_audio_set();

/* returns the blocksize of the device */
int get_blocksize();

int get_precision();
int get_samplerate();
int get_channels();

/* Returns the total number of fragments for the device */
int audio_fragstotal();

/* Queries the device in realtime to see how many frags are filled */
int fragments();

#endif
