/* compression.h */
/* Here we provide an interface to the compression routines that use ogg vorbis. */

#ifndef VAD_COMPRESSION
#define VAD_COMPRESSION

/* The max is arbitrary, we will set it higher than we should ever need       */
#define MAX_ENCODED_DATA_SIZE   5*MAX_BUFSIZE

/* Encoded Audio Data Object */
/* The results of the encoding process are stored here */
struct encoded_audio_data {
	int encoded_used;		     /* size of data after encoding   */
	int unencoded_used;		     /* size of data before encoding  */
	char data[MAX_ENCODED_DATA_SIZE];     /* The encoded audio data	      */ 	
	int sample_rate;		     /* sample rate of audio data     */
	int channels;			     /* number of channels of data    */
	int precision;			     /* Bits per sample               */	
};
/* set up the ogg vorbis encoding engine, called everytime a new audio */
/* param packet is recieved from the driver. If the function is	       */	
/* successful than it returns the number of bytes in the ogg headers   */
/* for the stream. The headers can be accessed with get_encoded_data.  */ 
int encode_init(struct audio_params *format, struct net_audio_params_pack *net_packet, float quality);

/* Free all the data structures and call the deinitialization functions for ogg vorbis */
void encode_deinit(struct net_audio_params_pack *net_packet);

/* Called to encode a buffer. The function will return 0 if data was written to */
/* the vorbis encoding engine but not enough was written to get a page. It will */
/* return the size of the ogg page when it has recieved engouh data. This means */
/* we can access the newly encoded data with get_encoded_data().		*/
int encode(char *buffer, int size);

/* Returns a pointer to an encoded_audio_data structure that holds the newly   */
/* compressed audio data that was a product of multiple calls to encode().     */
struct encoded_audio_data * get_encoded_data();

/* Has vorbis been setup previouslly */
int is_vorbis_setup();

/* Has encode() been called at all for this stream */
int is_new_stream();

#endif
