#ifndef VADSERV
#define VADSERV

#define DEFAULT_ADDR    "224.1.1.39"
#define PORT            60123 

#define MAX_BUFSIZE 	8192
#define MIN_BUFSIZE 	1
#define DEFAULT_BSIZE   1800 

#define SECS_TO_BUFFER  5		/* seconds of data we want the client to buffer */

#if defined (OPENBSD_VAD) 
#define DEFAULT_DEV     "/dev/svad0"	/* This is the slave device for the VAD on OpenBSd machines */
#include <sys/audioio.h>
#include "/sys/dev/vadio.h"		/* VAD data structures */
#endif

#if defined (LINUX_MAUDIO)
#include <sys/soundcard.h>
#define DEFAULT_DEV     "/dev/dsp2"	/* This is usually the slave device on Linux machines with maudio */
#endif

/* Included so we can use XDR for our data structures */
#include <rpc/rpc.h>


/* XDR routine for long long's. */
/* xdr_longlong and xdr_hyper are defined in some versions of XDR. */
/* This routine is pretty much a copy from those routines.	   */
static int xdr_long_long(xdrs, llp)
	XDR *xdrs;
	long long *llp;
{
	long l1, l2;

	/* If we are encoding */
	if(xdrs->x_op == XDR_ENCODE) {
		l1 = (long)((*llp) >> 32);
		l2 = (long)(*llp);
		return(XDR_PUTLONG(xdrs, &l1) && XDR_PUTLONG(xdrs, &l2));
	}

	/* If we are decoding */	
	else if(xdrs->x_op == XDR_DECODE) {
		if(XDR_GETLONG(xdrs, &l1) && XDR_GETLONG(xdrs, &l2)) {
			*llp = ((long long) l1) << 32;
			*llp |= l2;
			return(1);
		}
		else
			return(1);
	}

	else if(xdrs->x_op == XDR_FREE)
		return(1);
	
	else
		return(0);
			
}

/* Small structure to hold the format of audio data */
struct audio_params {
	unsigned long sample_rate;
	unsigned int  precision;
	unsigned int  channels;
};

/* This is the form of the packet sent at regular intervals to describe to */
/* audio format of the stream. All data stored in this structure will be   */
/* in network byte order.						   */
struct net_audio_params_pack  {
	char id[8]; 
	unsigned long long   	timestamp;
	unsigned long		end_of_stream;
	unsigned long  		sample_rate;
	unsigned long		precision;
	unsigned long  		channels;

	/* Ogg Vorbis Compression Data */
	char	       ogg_flag;	 
	unsigned int   serialno;
	ogg_packet     headers[3];

};

/* XDR Routine for ogg_packet structure */
static int xdr_ogg_packet(xdrs, op)
	XDR *xdrs;
	ogg_packet *op;
{

	/* If this is an ogg stream we must broadcast the headers too.  */
	/* They are stored in the three ogg_packet structures. However  */
	/* we can't just send over ogg_packet structures beacuse they   */
	/* are simply pointers to buffers of data in memory. So we   	*/
	/* append the real header data after each header structure.	*/ 
	
	/* Tf every routine works when encoding the fields */
	if(xdr_long(xdrs, &op->bytes) 		       &&
	   xdr_long(xdrs, &op->b_o_s)                  &&
	   xdr_long(xdrs, &op->e_o_s)		       &&
	   xdr_long_long(xdrs, &op->granulepos)        &&
	   xdr_long_long(xdrs, &op->packetno)	       &&
	   xdr_opaque(xdrs, (char*)op->packet, op->bytes) )
		return(1);

	/* If any of the primitives fails, we fail */			
	return(0);
	
}
	
/* XDR Routine for net_audio_params_pack */
static int xdr_net_audio_params_pack(xdrs, np)
	XDR *xdrs;
	struct net_audio_params_pack *np;
{
	if(xdr_opaque(xdrs, (char*)&np->id, 8)          &&
	   xdr_long_long(xdrs, &np->timestamp) 	 	&&
	   xdr_u_long(xdrs, &np->end_of_stream)		&&
	   xdr_u_long(xdrs, &np->sample_rate) 	  	&&
	   xdr_u_long(xdrs, &np->precision)    	  	&&
	   xdr_u_long(xdrs, &np->channels)     	  	&&
	   xdr_u_int(xdrs,  &np->serialno)		&&
	   xdr_u_char(xdrs, &np->ogg_flag))  {      	
		
		/* If this is an ogg stream, get the headers too */
		if(np->ogg_flag == 1) { 
		
			if(xdr_ogg_packet(xdrs, &np->headers[0]) &&
	   		   xdr_ogg_packet(xdrs, &np->headers[1]) &&
	   		   xdr_ogg_packet(xdrs, &np->headers[2])   )  
				return(1);				     
		
		}
		
		/* Otherwise, we still have a good packet */
		else		       	
			return(1);
	}
	
	/* If any routine failed, the whole thing has failed */	
	return(0);  /* Failure */
	
}

/* structure for the header of the audio_data_packet */
struct audio_data_packet_hdr {

	/* Length of the data portion of our packet */
	unsigned long len;
	
	/* timestamp is a very important parameter of the audio_data_packet.  */
	/* This is not a timestamp of when the packet was sent, it is a time  */
	/* at which this packet should be played. The clients used this value */
	/* along with the current time that is sent with each net params pack */
	/* to decide whether they are going to slow or two fast.	      */
	unsigned long long timestamp;
	
};

/* XDR Routine to read and convert audio_data_packet_hdr's */
static int xdr_audio_data_packet_hdr(xdrs, hdr)
	        XDR *xdrs;
		        struct audio_data_packet_hdr *hdr;
{
	if(xdr_u_long(xdrs, &hdr->len)                  &&
	   xdr_long_long(xdrs, &hdr->timestamp)) 
		return(1);
	
	return(0);

}


/* This is the format of the audio data packet */
struct audio_data_packet  {

	/* Header */
	struct audio_data_packet_hdr hdr;
	
	/* This buffer holds the audio data. We probably won't use it all so  */
	/* so when we send the packet we WONT specify the size of the message */
	/* with something like sizeof(struct audio_data_packet).	      */
	char data[MAX_BUFSIZE];

};

/* XDR Routine to read and convert audio_data_packet's */
static int xdr_audio_data_packet(xdrs, ap) 
	XDR *xdrs;
	struct audio_data_packet *ap;
{
	if(xdr_audio_data_packet_hdr(xdrs, &ap->hdr)	    &&
	   xdr_opaque(xdrs, (char*)&ap->data, ap->hdr.len))
		return(1);

	return(0);	

}


/* Grabbed these from, http://www.codeproject.com/cpp/endianness.asp By Runner */
#define ntohll(x) (((long long)(ntohl((int)((x << 32) >> 32))) << 32) | (unsigned int)ntohl(((int)(x >> 32))))
#define htonll(x) ntohll(x)



#endif
